export declare function fn(arg: string): string;
export declare namespace fn {
  var staticProp: string;
}

export declare function unused(arg: string): string;
export declare namespace unused {
  var staticProp: string;
}
