/// <reference types="react" />
declare type JSXElements$1 = keyof JSX.IntrinsicElements;
declare const a: JSXElements$1[];
declare type JSXElements = keyof JSX.IntrinsicElements;
declare const b: JSXElements[];
export { a, b };
