// @ts-check
/** @type {import('../../testcases').Meta} */
export default {
  options: {},
  rollupOptions: {
    input: {
      a: "main-a.ts",
      b: "main-b.tsx",
    },
  },
  rollupVersion: "4.44.0",
};
