export { ReactElement } from 'react';
// Last module patch version validated against: 3.0.0
/**
 * Container element type usable for mouse/touch functions
 */
type DragContainerElement = HTMLElement | SVGSVGElement | SVGGElement;
export type { DragContainerElement };
