// @ts-check
/** @type {import('../../testcases').Meta} */
export default {
  options: {},
  // `./Component` resolves to Component.tsx (Rollup module ids keep the
  // original .tsx path) — the specifier must be rewritten, with no warning
  expectedWarnings: [],
};
