declare const hstack: string;
declare const vstack: string;
declare const visuallyHidden: string;
declare var export_default = [
    hstack,
    vstack,
    visuallyHidden
];
export { export_default as default };
