declare const sLit = "";
declare const nLit = 0;
declare const aLit: never[];
declare const sLitDef: "";
declare const nLitDef: 0;
declare const sDef: string;
declare const nDef: number;
declare const aDef: Array<number>;
declare const tuple: [number, string];
declare const unique: unique symbol;
export { aDef, aLit, nDef, nLit, nLitDef, sDef, sLit, sLitDef, tuple, unique };
