/// <reference types="react" />
declare type JSXElements = keyof JSX.IntrinsicElements;
declare const a: JSXElements[];
export { JSXElements, a };
export { B } from "./common";
export { A } from "./ref-from-a";
