/**
 * A function with doc-comment that is imported first
 */
declare function first(): void;
/**
 * A function with doc-comment that is imported second
 */
declare function second(): void;
/**
 * A function with doc-comment in the main file
 */
declare function main(): void;
export { first, main, second };
