export default function (): number;
export default function (constructor: Function): void;
export default function (name: string, descriptor: PropertyDescriptor): string;

// conflicting name:
export declare const export_default: number;
