export declare const A: "string";
export declare const B: 8;
export declare const C: void;
export declare const D: unknown;
export declare const E: any;
export declare const F: boolean;
export declare const G: number;
export declare const H: string;
export declare const I: object;
export declare const J: null;
export declare const K: undefined;
export declare const L: symbol;
export declare const M: never;
export interface N {
  foo(): this;
}
export declare const O: bigint;
