export type User = {
    id: string;
    name: string;
};
//# sourceMappingURL=data:application/json,%7B%22version%22%3A3%2C%22file%22%3A%22inline-urlencoded.d.ts%22%2C%22sourceRoot%22%3A%22%22%2C%22sources%22%3A%5B%22..%2Fsrc%2Findex.ts%22%5D%2C%22names%22%3A%5B%5D%2C%22mappings%22%3A%22AAAA%2CMAAM%2CMAAM%2CIAAI%2CGAAG%3BIAClB%2CEAAE%2CEAAE%2CMAAM%2CCAAC%3BIACX%2CIAAI%2CEAAE%2CMAAM%2CCAAC%3BCACb%2CCAAC%22%7D
