declare global {
  interface Window {
    __TEST__: string;
  }
}
// See //# sourceMappingURL=generated.map for details
declare const value: string;
declare const result: typeof value;
export { result };
