declare var a: boolean /*comment here*/;
declare var /* comment there */ b: string;
declare let c: string;
declare let d: number;
// prettier-ignore
declare const foo: number;
declare const bar: string;

export { c, d, foo, bar };
