// @ts-check

/** @type {import('../../testcases').Meta} */
export default {
  options: {},
  rollupOptions: {},
  /**
   * TODO: Until we found a safe way to handle the type-only import/export, 
   * we skip this test.
   */
  skip: true,
};
