// @ts-check
/** @type {import('../../testcases').Meta} */
export default {
  options: {
    respectExternal: true,
  },
  rollupOptions: {
    external: ["react"],
  },
  rollupVersion: '3.25.0',
};
