export declare const config: {
  normalize: (inVar: import("./settings").In) => import("./settings").Out;
};

export declare const options: {
  normalize: (inVar: import("./settings").In) => import("./settings").Out;
};

export declare const params: {
  normalize: (inVar: import("./settings").In) => import("./settings").Out;
};
